@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::SOAP;
 use vars '$VERSION';
-$VERSION = '2.19';
+$VERSION = '2.20';
 
 
 use Log::Report 'xml-compile-soap', syntax => 'SHORT';
@@ -251,7 +251,7 @@ sub _writer_header($)
         my $element = $part->{element};
         my $code    = $part->{writer}
          || $self->_writer($element, %$args, elements_qualified => 'TOP'
-              , include_namespaces => sub {$_[0] ne $soapenv});
+              , include_namespaces => sub {$_[0] ne $soapenv && $_[2]});
 
         push @rules, $label => $code;
         push @hlabels, $label;
@@ -301,7 +301,7 @@ sub _writer_body_element($$)
 
     $part->{writer}
        ||= $self->_writer($element, %$args, elements_qualified => 'TOP'
-            , include_namespaces => sub {$_[0] ne $soapenv});
+            , include_namespaces => sub {$_[0] ne $soapenv && $_[2]});
 }
 
 sub _writer_body_type($$)
@@ -320,7 +320,7 @@ sub _writer_body_type($$)
         $self->schemas->compileType
           ( WRITER  => $part->{type}, %$args
           , element => $part->{name}
-          , include_namespaces => sub {$_[0] ne $soapenv}
+          , include_namespaces => sub {$_[0] ne $soapenv && $_[2]}
           );
 }
 
